#version 430

layout(binding = 0, std140) uniform UBO
{
    mat4 MVP;
} global;

struct Push
{
    vec4 SourceSize;
    vec4 OriginalSize;
    vec4 OutputSize;
};

uniform Push registers;

layout(binding = 4) uniform sampler2D Original;
layout(binding = 2) uniform sampler2D Source;
layout(binding = 3) uniform sampler2D LUT;

layout(location = 0) in vec2 vTexCoord;
layout(location = 0) out vec4 FragColor;

void main()
{
    vec2 fp = fract(vTexCoord * registers.SourceSize.xy);
    vec2 quad = sign(vec2(-0.5) + fp);
    float dx = registers.SourceSize.z;
    float dy = registers.SourceSize.w;
    vec3 p1 = texture(Original, vTexCoord).xyz;
    vec3 p2 = texture(Original, vTexCoord + (vec2(dx, dy) * quad)).xyz;
    vec3 p3 = texture(Original, vTexCoord + (vec2(dx, 0.0) * quad)).xyz;
    vec3 p4 = texture(Original, vTexCoord + (vec2(0.0, dy) * quad)).xyz;
    vec3 pixels[4];
    pixels[0] = p1;
    pixels[1] = p2;
    pixels[2] = p3;
    pixels[3] = p4;
    vec2 index = texture(Source, vTexCoord).xy * vec2(255.0, 240.0);
    index.y += dot(floor(fp * 4.0), vec2(1.0, 4.0));
    vec2 _step = vec2(0.00390625);
    vec2 offset = _step / vec2(2.0);
    vec4 weights = texture(LUT, (index * _step) + offset);
    float sum = dot(weights, vec4(1.0));
    vec4 tmp = vec4((weights / vec4(sum)).x, (weights / vec4(sum)).y, (weights / vec4(sum)).z, (weights / vec4(sum)).w);
    vec3 res = pixels[0] * tmp.x;
    res += (pixels[1] * tmp.y);
    res += (pixels[2] * tmp.z);
    res += (pixels[3] * tmp.w);
    FragColor = vec4(res, 1.0);
}

